import React from 'react';
import { applyPolyfills, defineCustomElements } from '@core-design/components-web/loader';

function attachEventProps(node, newProps, oldProps = {}) {
    const className = getClassName(node.classList, newProps, oldProps);
    if (className) {
        node.className = className;
    }
    Object.keys(newProps).forEach(name => {
        if (name === 'children' || name === 'style' || name === 'ref' || name === 'className') {
            return;
        }
        if (name.indexOf('on') === 0 && name[2] === name[2].toUpperCase()) {
            const eventName = name.substring(2);
            const eventNameLc = eventName[0].toLowerCase() + eventName.substring(1);
            if (!isCoveredByReact(eventNameLc)) {
                syncEvent(node, eventNameLc, newProps[name]);
            }
        }
        else {
            node[name] = newProps[name];
        }
    });
}
function getClassName(classList, newProps, oldProps) {
    // map the classes to Maps for performance
    const currentClasses = arrayToMap(classList);
    const incomingPropClasses = arrayToMap(newProps.className ? newProps.className.split(' ') : []);
    const oldPropClasses = arrayToMap(oldProps.className ? oldProps.className.split(' ') : []);
    const finalClassNames = [];
    // loop through each of the current classes on the component
    // to see if it should be a part of the classNames added
    currentClasses.forEach(currentClass => {
        if (incomingPropClasses.has(currentClass)) {
            // add it as its already included in classnames coming in from newProps
            finalClassNames.push(currentClass);
            incomingPropClasses.delete(currentClass);
        }
        else if (!oldPropClasses.has(currentClass)) {
            // add it as it has NOT been removed by user
            finalClassNames.push(currentClass);
        }
    });
    incomingPropClasses.forEach(s => finalClassNames.push(s));
    return finalClassNames.join(' ');
}
/**
 * Checks if an event is supported in the current execution environment.
 * @license Modernizr 3.0.0pre (Custom Build) | MIT
 */
function isCoveredByReact(eventNameSuffix, doc = document) {
    const eventName = 'on' + eventNameSuffix;
    let isSupported = eventName in doc;
    if (!isSupported) {
        const element = doc.createElement('div');
        element.setAttribute(eventName, 'return;');
        isSupported = typeof element[eventName] === 'function';
    }
    return isSupported;
}
function syncEvent(node, eventName, newEventHandler) {
    const eventStore = node.__events || (node.__events = {});
    const oldEventHandler = eventStore[eventName];
    // Remove old listener so they don't double up.
    if (oldEventHandler) {
        node.removeEventListener(eventName, oldEventHandler);
    }
    if (newEventHandler != null) {
        // Bind new listener.
        node.addEventListener(eventName, (eventStore[eventName] = function handler(e) {
            newEventHandler.call(this, e);
        }));
    }
}
function arrayToMap(arr) {
    const map = new Map();
    arr.forEach((s) => map.set(s, s));
    return map;
}

const dashToPascalCase = (str) => str
    .toLowerCase()
    .split('-')
    .map(segment => segment.charAt(0).toUpperCase() + segment.slice(1))
    .join('');
const createForwardRef = (ReactComponent, displayName) => {
    const forwardRef = (props, ref) => {
        return React.createElement(ReactComponent, Object.assign({}, props, { forwardedRef: ref }));
    };
    forwardRef.displayName = displayName;
    return React.forwardRef(forwardRef);
};

var __rest = (undefined && undefined.__rest) || function (s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
            if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))
                t[p[i]] = s[p[i]];
        }
    return t;
};
const createReactComponent = (tagName) => {
    const displayName = dashToPascalCase(tagName);
    const ReactComponent = class extends React.Component {
        constructor(props) {
            super(props);
            this.ref = React.createRef();
        }
        componentDidMount() {
            this.componentDidUpdate(this.props);
        }
        componentDidUpdate(prevProps) {
            const node = this.ref.current;
            attachEventProps(node, this.props, prevProps);
        }
        render() {
            const _a = this.props, { children, forwardedRef, style, className, ref } = _a, cProps = __rest(_a, ["children", "forwardedRef", "style", "className", "ref"]);
            const propsToPass = Object.keys(cProps).reduce((acc, name) => {
                const isEventProp = name.indexOf('on') === 0 && name[2] === name[2].toUpperCase();
                const isDataProp = name.indexOf('data-') === 0;
                const isAriaProp = name.indexOf('aria-') === 0;
                if (isEventProp) {
                    const eventName = name.substring(2).toLowerCase();
                    if (typeof document !== "undefined" && isCoveredByReact(eventName)) {
                        acc[name] = cProps[name];
                    }
                }
                else if (isDataProp || isAriaProp) {
                    acc[name] = cProps[name];
                }
                return acc;
            }, {});
            const newProps = Object.assign(Object.assign({}, propsToPass), { ref: this.ref, style,
                className });
            return React.createElement(tagName, newProps, children);
        }
        static get displayName() {
            return displayName;
        }
    };
    return createForwardRef(ReactComponent, displayName);
};

/* eslint-disable */
applyPolyfills().then(() => defineCustomElements());
const CoreAlert = /*@__PURE__*/ createReactComponent('core-alert');
const CoreAvatar = /*@__PURE__*/ createReactComponent('core-avatar');
const CoreBadge = /*@__PURE__*/ createReactComponent('core-badge');
const CoreButton = /*@__PURE__*/ createReactComponent('core-button');
const CoreCheckbox = /*@__PURE__*/ createReactComponent('core-checkbox');
const CoreColorGrid = /*@__PURE__*/ createReactComponent('core-color-grid');
const CoreDropdown = /*@__PURE__*/ createReactComponent('core-dropdown');
const CoreDropdownItem = /*@__PURE__*/ createReactComponent('core-dropdown-item');
const CoreIcon = /*@__PURE__*/ createReactComponent('core-icon');
const CoreIconGrid = /*@__PURE__*/ createReactComponent('core-icon-grid');
const CoreInput = /*@__PURE__*/ createReactComponent('core-input');
const CoreLabel = /*@__PURE__*/ createReactComponent('core-label');
const CorePopover = /*@__PURE__*/ createReactComponent('core-popover');
const CoreProgress = /*@__PURE__*/ createReactComponent('core-progress');
const CoreRadio = /*@__PURE__*/ createReactComponent('core-radio');
const CoreRadioGroup = /*@__PURE__*/ createReactComponent('core-radio-group');
const CoreSelect = /*@__PURE__*/ createReactComponent('core-select');
const CoreSpinner = /*@__PURE__*/ createReactComponent('core-spinner');
const CoreTab = /*@__PURE__*/ createReactComponent('core-tab');
const CoreTabGroup = /*@__PURE__*/ createReactComponent('core-tab-group');
const CoreTag = /*@__PURE__*/ createReactComponent('core-tag');
const CoreTextarea = /*@__PURE__*/ createReactComponent('core-textarea');
const CoreToast = /*@__PURE__*/ createReactComponent('core-toast');
const CoreToggle = /*@__PURE__*/ createReactComponent('core-toggle');
const CoreTooltip = /*@__PURE__*/ createReactComponent('core-tooltip');

export { CoreAlert, CoreAvatar, CoreBadge, CoreButton, CoreCheckbox, CoreColorGrid, CoreDropdown, CoreDropdownItem, CoreIcon, CoreIconGrid, CoreInput, CoreLabel, CorePopover, CoreProgress, CoreRadio, CoreRadioGroup, CoreSelect, CoreSpinner, CoreTab, CoreTabGroup, CoreTag, CoreTextarea, CoreToast, CoreToggle, CoreTooltip };
//# sourceMappingURL=index.esm.js.map
